/*
 *  guid.c
 *  
 *
 *  Created by Marco Bambini on 8/18/08.
 *  Copyright 2005-2008 REALSoftware, Inc. All rights reserved.
 *
 */

/*
	From Wikipedia (http://en.wikipedia.org/wiki/Globally_Unique_Identifier):
 
	A Universally Unique Identifier (UUID) is an identifier standard used in software construction,
	standardized by the Open Software Foundation (OSF) as part of the Distributed Computing Environment (DCE).
	The intent of UUIDs is to enable distributed systems to uniquely identify information without
	significant central coordination. Thus, anyone can create a UUID and use it to identify something with reasonable
	confidence that the identifier will never be unintentionally used by anyone for anything else.
	Information labeled with UUIDs can therefore be later combined into a single database without needing to resolve
	name conflicts.
 
	A UUID is a 16-byte (128-bit) number. The number of theoretically possible UUIDs is therefore about 3.4 × 10 pow 38.
	This means that 1 trillion UUIDs would have to be created every nanosecond for 10 billion years to exhaust
	the number of UUIDs.
 
 */

#include "guid.h"

#define PLUGIN_NAME			"GUID"
#define PLUGIN_VERSION		"1.0"
#define PLUGIN_COPYRIGHT	"(c) 2008 REALSoftware, Inc"
#define PLUGIN_DESCRIPTION	"This is a sample plugin that adds the GUID functions to the REAL SQL Server. More on the GUID function available at: http://en.wikipedia.org/wiki/Globally_Unique_Identifier"

void rsql_plugin_init (void)
{
	// Nothing to allocate/prepare at startup
	return;
}

void rsql_plugin_exit (void)
{
	// Nothing to free on exit
	return;
}

void rsql_plugin_info (char **name, char **version, char **copyright, char **description)
{	
	// Returns plugins informatio
	SET_PLUGIN_NAME(PLUGIN_NAME);
	SET_PLUGIN_VERSION(PLUGIN_VERSION);
	SET_PLUGIN_COPYRIGHT(PLUGIN_COPYRIGHT);
	SET_PLUGIN_DESCRIPTION(PLUGIN_DESCRIPTION);
}

void rsql_plugin_register (void *pluginref)
{	
	// Some DBMS implementations uses GUID while others uses UUID, in this way you can use both
	rsql_register_sql_function(pluginref, "GUID", 0, RSQL_ANY, NULL, generate_guid);
	rsql_register_sql_function(pluginref, "UUID", 0, RSQL_ANY, NULL, generate_guid);
}

void generate_guid (rsql_context *ctx, int nparams, rsql_value **value)
{
	// Core plugin's function	
	#ifdef RSQL_MAC
	CFUUIDRef	guidref;
	CFStringRef	guidstr;
	char		buffer[256];
	
	guidref = CFUUIDCreate(NULL);
	guidstr = CFUUIDCreateString(NULL, guidref);
	CFStringGetCString(guidstr, buffer, 256, kCFStringEncodingMacRoman);
	
	rsql_result_text(ctx, buffer, RSQL_ZEROTERM, RSQL_TRANSIENT);
	
	CFRelease(guidref);
	CFRelease(guidstr);
	#endif
	
	#ifdef RSQL_WINDOWS
	UUID			pUUID;
	unsigned char	*buffer;
	
	if (UuidCreate(&pUUID) != RPC_S_OK) goto abort_plugin;
	if (UuidToStringA(&pUUID, &buffer) != RPC_S_OK) goto abort_plugin;
	
	rsql_result_text(ctx, buffer, RSQL_ZEROTERM, RSQL_TRANSIENT);
	
	RpcStringFreeA(&buffer);
	#endif

	#ifdef RSQL_LINUX
	uuid_t	uuid;
    char	buffer[MAXGUIDLEN+1];
	
	uuid_generate(uuid);
	uuid_unparse(uuid, buffer);
	
	rsql_result_text(ctx, buffer, RSQL_ZEROTERM, RSQL_TRANSIENT);
	#endif
	
	return;
	
abort_plugin:
	rsql_result_error(ctx, "Unable to generate a GUID!", RSQL_ZEROTERM);
	return;
}
